//	Most of the functions of this file came from MD2 Viewer developped by Mete Ciragan

#ifdef ZDOOM_GL
//#ifdef DOOM_GL
//#if defined(DOOM_GL) || defined (GL_HEXEN)

#include <stdio.h>
#include <stdlib.h>
#include <string.h> /* memset */
#include <math.h> /* sqrt */
#include <windows.h>
#include <gl/gl.h>
#include "md2.h"
#ifdef GL_HEXEN
#include "h2def.h"
#elif DOOM_GL
#include "doomtype.h"
#else
#include "doomdef.h"
#endif

#include "I_system.h"
#include "gl_struct.h"

int STACK_ARGS ConPrintf (const char *format, ...);
#define printf ConPrintf

#define NUMVERTEXNORMALS 162

float avertexnormals[NUMVERTEXNORMALS][3] = {
#include "anorms.h"
};

static int g_glcmds = 1; /* use glcommands */
static int g_interp = 1; /* interpolate frames */

extern float MAP_COEFF;
extern int (WINAPI *glColorTableEXT)(GLenum enum1,GLenum enum2,GLsizei size,GLenum enum3,GLenum enum4,const GLvoid *data);

extern PROC glGetColorTableParameterivEXT;

#define word short
typedef struct
{
    byte manufacturer;   /* 10 = ZSoft .pcx */
    byte version;        /* 0 = Version 2.5 of PC Paintbrush 
                            2 = Version 2.8 w/palette information 
                            3 = Version 2.8 w/o palette information 
                            4 = PC Paintbrush for Windows(Plus for
                                Windows uses Ver 5) 
                            5 = Version 3.0 and > of PC Paintbrush
                                and PC Paintbrush +, includes
                                Publisher's Paintbrush . Includes
                                24-bit .PCX files */
    byte encoding;       /* 1 = .pcx rle encoding */
    byte bitsPerPixel;   /* 1, 2, 4, 8 per plane */
    word xmin;
    word ymin;
    word xmax;
    word ymax;
    word hDpi;
    word vDpi;
    byte colorMap[48];
    byte reserved;       /* should be set to 0 */
    byte numPlanes;      /* number of color planes */
    word bytesPerLine;   /* MUST be EVEN number */
    word paletteInfo;    /* 1 = color, 2 = grayscale */
    word hScreenSize;  
    word vScreenSize;
    byte filler[54];     /* set all to 0 */
} mxPcxHeader;

extern int xS[512],yS[512];
void fn_vStretchTexture(long lWidthS,long lHeightS,long lWidthD,long lHeightD);
int fn_iGetTexHeightGL(int iHeight);
int g_iModel=0;
int *g_a_iTexMD2,g_iTex=0;
extern GLTexture *p_stMD2Tex;

extern byte *p_bPalette;
extern GLboolean g_bPaletteTexture;

EXTERN_CVAR(g_bUseMipMap)	// MR3108
void MD2_vLoadTexturePCX(char *filename)
{	unsigned char palette[768];
	unsigned char *image;
	unsigned char *image2;
	mxPcxHeader header;
	int w,h;
	int x,y;
    int ptr = 0;
	int ch, rep;
	int size;
    FILE *file = fopen (filename, "rb");
    if (!file)
        return;

    if (fread (&header, sizeof (mxPcxHeader), 1, file) == -1)
	{
        fclose (file);
        return;
    }
	(void) fseek (file, -768, SEEK_END);

    w = header.xmax - header.xmin + 1;
    h = header.ymax - header.ymin + 1;

	image=(unsigned char *)malloc(w*h);

    if (fread ((byte *) palette, sizeof (byte), 768, file) == -1)
	{
        fclose (file);
        return;
    }

    (void) fseek(file, sizeof (mxPcxHeader), SEEK_SET);
	
	size= w * h;

    while (ptr < size)
	{
        ch = fgetc(file);
        if (ch >= 192)
		{
            rep = ch - 192;
            ch = fgetc(file);
        }
        else {
            rep = 1;
        }

        while (rep--)
			image[ptr++] = ch;
    }

    fclose(file);

	// MR2306
	/*iGLTexWidth=fn_iGetTexHeightGL(w);
	iGLTexHeight=fn_iGetTexHeightGL(h);
	fn_vStretchTexture(w,h,iGLTexWidth,iGLTexHeight);*/
	
	image2=(unsigned char *)malloc(w*h*3);
	for (x=0;x<w;x++)
	for (y=0;y<h;y++)
	{	image2[(x+y*w)*3]=palette[3*image[x+y*w]];
		image2[(x+y*w)*3+1]=palette[3*image[x+y*w]+1];
		image2[(x+y*w)*3+2]=palette[3*image[x+y*w]+2];
	}
	/*for (x=0;x<iGLTexWidth;x++)
	for (y=0;y<iGLTexHeight;y++)
	{	image2[(x+y*iGLTexWidth)*3]=palette[3*image[xS[x]+yS[y]*w]];
		image2[(x+y*iGLTexWidth)*3+1]=palette[3*image[xS[x]+yS[y]*w]+1];
		image2[(x+y*iGLTexWidth)*3+2]=palette[3*image[xS[x]+yS[y]*w]+2];
	}*/

	//glBindTexture (GL_TEXTURE_2D, 50000+g_iModel++);
	p_stMD2Tex[g_iTex].iWidth=w;
	p_stMD2Tex[g_iTex].iHeight=h;
	glBindTexture (GL_TEXTURE_2D, 50000+g_iTex++);
	glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	if (g_bUseMipMap.value)		// MR3108
	{
		glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, g_bUseMipMap.value==1?(float)GL_LINEAR_MIPMAP_NEAREST:(float)GL_LINEAR_MIPMAP_LINEAR);
		glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, g_bUseMipMap.value==1?(float)GL_LINEAR_MIPMAP_NEAREST:(float)GL_LINEAR_MIPMAP_LINEAR);
	}
	else
	{	glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	}
	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	if (g_bPaletteTexture)
	{	glColorTableEXT(GL_TEXTURE_2D,GL_RGB8,256,GL_RGB,GL_UNSIGNED_BYTE,p_bPalette);
		if (g_bUseMipMap.value)		// MR3108
			gluBuild2DMipmaps(GL_TEXTURE_2D, 1, w, h, GL_COLOR_INDEX, GL_UNSIGNED_BYTE, image);
		else
			glTexImage2D(GL_TEXTURE_2D, 0, GL_COLOR_INDEX8_EXT, w, h, 0, GL_COLOR_INDEX, GL_UNSIGNED_BYTE, image);
		p_stMD2Tex[g_iTex-1].p_bRGBBuffer=image;
		free(image2);
	}
	else
	{	if (g_bUseMipMap.value)		// MR3108
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, w, h, GL_RGB, GL_UNSIGNED_BYTE, image2);
		else
			glTexImage2D (GL_TEXTURE_2D, 0, GL_RGB, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, image2);
		p_stMD2Tex[g_iTex-1].p_bRGBBuffer=image2;
		free(image);
	}
}

// MR
//#ifdef DOOM_GL
#if 1
float *fMD2Scale;
/*float fMD2Scale[NB_MD2]=
{	
	//110.0f,		// imp.md2
	100.0f,		// imp.md2
	110.0f,		// soldier2.md2
	110.0f,		// sargeant.md2
	90.0f,		// pinky.md2
	110.0f,		// baron.md2
	110.0f,		// qcaco.md2
	110.0f,		// cyber.md2
	80.0f,		// spider.md2
	110.0f,		// chainer.md2
	110.0f,		// arach.md2
	110.0f,		// archv.md2
	110.0f,		// macum.md2
	80.0f		// revenant.md2
};*/
#else
float fMD2Scale[NB_MD2]=
{	
	200.0f		// mal.md2
};
#endif

/*
 * load model
 */
md2_model_t *md2_readModel (const char *filename)
{
	FILE *file;
	md2_model_t *model;
	byte buffer[MD2_MAX_FRAMESIZE];
	int i,k;

	k=strlen(filename);
	while ((filename[k]!='/')&&(filename[k]!='\\')&&(k>=0)) k--;
	k++;

	model = (md2_model_t *) malloc (sizeof (md2_model_t));
	if (!model)
		return 0;

	file = fopen (filename, "rb");
	if (!file)
	{
		if (strcmp(filename,"-"))
			printf("Can't load Model %s\n",filename);
		free (model);
		return 0;
	}

	/* initialize model and read header */
	memset (model, 0, sizeof (md2_model_t));
	fread (&model->header, sizeof (md2_header_t), 1, file);
	if (model->header.magic != (int) (('2' << 24) + ('P' << 16) + ('D' << 8) + 'I'))
	{
		fclose (file);
		free (model);
		return 0;
	}

	/* read skins */
	fseek (file, model->header.offsetSkins, SEEK_SET);
	if (model->header.numSkins > 0)
	{
		model->skins = (md2_skin_t *) malloc (sizeof (md2_skin_t) * model->header.numSkins);
		if (!model->skins)
		{
			md2_freeModel (model);
			return 0;
		}

		for (i = 0; i < model->header.numSkins; i++)
		{ char name[64];
		  int j;

			fread (&model->skins[i], sizeof (md2_skin_t), 1, file);
			// MR
			//strcpy(name,"md2/");
			strncpy(name,filename,k);
			name[k]=0;
			j=strlen(model->skins[i]);
			while ((model->skins[i][j]!='/')&&(j>=0)) j--;
			j++;
			strcat(name,model->skins[i]+j);
			// MR (anti-bug)
			if (!i)	// only one skin...
				MD2_vLoadTexturePCX(name);
		}
	}

	/* read texture coordinates */
	fseek (file, model->header.offsetTexCoords, SEEK_SET);
	if (model->header.numTexCoords > 0)
	{
		model->texCoords = (md2_textureCoordinate_t *) malloc (sizeof (md2_textureCoordinate_t) * model->header.numTexCoords);
		if (!model->texCoords)
		{
			md2_freeModel (model);
			return 0;
		}

		for (i = 0; i < model->header.numTexCoords; i++)
			fread (&model->texCoords[i], sizeof (md2_textureCoordinate_t), 1, file);
	}

	/* read triangles */
	fseek (file, model->header.offsetTriangles, SEEK_SET);
	if (model->header.numTriangles > 0)
	{
		model->triangles = (md2_triangle_t *) malloc (sizeof (md2_triangle_t) * model->header.numTriangles);
		if (!model->triangles)
		{
			md2_freeModel (model);
			return 0;
		}

		for (i = 0; i < model->header.numTriangles; i++)
			fread (&model->triangles[i], sizeof (md2_triangle_t), 1, file);
	}

	/* read alias frames */
	fseek (file, model->header.offsetFrames, SEEK_SET);
	if (model->header.numFrames > 0)
	{
		model->frames = (md2_frame_t *) malloc (sizeof (md2_frame_t) * model->header.numFrames);
		if (!model->frames)
		{
			md2_freeModel (model);
			return 0;
		}

		for (i = 0; i < model->header.numFrames; i++)
		{
			md2_alias_frame_t *frame = (md2_alias_frame_t *) buffer;
			int j;

			model->frames[i].vertices = (md2_triangleVertex_t *) malloc (sizeof (md2_triangleVertex_t) * model->header.numVertices);
			if (!model->frames[i].vertices)
			{
				md2_freeModel (model);
				return 0;
			}

			fread (frame, 1, model->header.frameSize, file);
			strcpy (model->frames[i].name, frame->name);
			for (j = 0; j < model->header.numVertices; j++)
			{
				model->frames[i].vertices[j].vertex[0] = (float) ((int) frame->alias_vertices[j].vertex[0]) * frame->scale[0] + frame->translate[0];
				model->frames[i].vertices[j].vertex[2] = -1* ((float) ((int) frame->alias_vertices[j].vertex[1]) * frame->scale[1] + frame->translate[1]);
				model->frames[i].vertices[j].vertex[1] = (float) ((int) frame->alias_vertices[j].vertex[2]) * frame->scale[2] + frame->translate[2];
// MR
				/*model->frames[i].vertices[j].vertex[0] /= fMD2Scale[g_iModel-1];	// 110
				model->frames[i].vertices[j].vertex[2] /= fMD2Scale[g_iModel-1];
				model->frames[i].vertices[j].vertex[1] /= fMD2Scale[g_iModel-1];*/
				model->frames[i].vertices[j].vertex[0] /= fMD2Scale[g_iModel];	// 110
				model->frames[i].vertices[j].vertex[2] /= fMD2Scale[g_iModel];
				model->frames[i].vertices[j].vertex[1] /= fMD2Scale[g_iModel];

				model->frames[i].vertices[j].normal[0] = avertexnormals[frame->alias_vertices[j].lightNormalIndex][0];
				model->frames[i].vertices[j].normal[1] = avertexnormals[frame->alias_vertices[j].lightNormalIndex][1];
				model->frames[i].vertices[j].normal[2] = avertexnormals[frame->alias_vertices[j].lightNormalIndex][2];
				model->frames[i].vertices[j].lightNormalIndex = frame->alias_vertices[j].lightNormalIndex;
			}
		}
	}

	/* read gl commands */
	fseek (file, model->header.offsetGlCommands, SEEK_SET);
	if (model->header.numGlCommands)
	{
		model->glCommandBuffer = (int *) malloc (sizeof (int) * model->header.numGlCommands);
		if (!model->glCommandBuffer)
		{
			md2_freeModel (model);
			return 0;
		}

		fread (model->glCommandBuffer, sizeof (int), model->header.numGlCommands, file);
	}

	fclose (file);

	return model;
}



/*
 * free model
 */
void md2_freeModel (md2_model_t *model)
{
	if (model)
	{
		if (model->skins)
			free (model->skins);

		if (model->texCoords)
			free (model->texCoords);

		if (model->triangles)
			free (model->triangles);

		if (model->frames)
		{
			int i;

			for (i = 0; i < model->header.numFrames; i++)
			{
				if (model->frames[i].vertices)
					free (model->frames[i].vertices);
			}
			free (model->frames);
		}

		if (model->glCommandBuffer)
			free (model->glCommandBuffer);

		free (model);
	}
}



/*
 * set draw style, either with glcommands, interpolated
 */
void md2_setStyle (int glcmds, int interp)
{
	g_glcmds = glcmds;
	g_interp = interp;
}



/*
 * center model
 *
 */
void md2_getBoundingBox (md2_model_t *model, float *minmax)
{
	int i;
	float minx, maxx;
	float miny, maxy;
	float minz, maxz;

	minx = miny = minz = 999999.0f;
	maxx = maxy = maxz = -999999.0f;

	/* get bounding box */
	for (i = 0; i < model->header.numVertices; i++)
	{
		md2_triangleVertex_t *v = &model->frames[0].vertices[i];

		if (v->vertex[0] < minx)
			minx = v->vertex[0];
		else if (v->vertex[0] > maxx)
			maxx = v->vertex[0];

		if (v->vertex[1] < miny)
			miny = v->vertex[1];
		else if (v->vertex[1] > maxy)
			maxy = v->vertex[1];

		if (v->vertex[2] < minz)
			minz = v->vertex[2];
		else if (v->vertex[2] > maxz)
			maxz = v->vertex[2];
	}

	minmax[0] = minx;
	minmax[1] = maxx;
	minmax[2] = miny;
	minmax[3] = maxy;
	minmax[4] = minz;
	minmax[5] = maxz;
}

CVAR (g_iMD2VLightDist, "100", CVAR_ARCHIVE)		// MR231200
EXTERN_CVAR (g_bMD2Light)
float cosA,sinA,xC,yC,zC/*,LightLev*/;	// MR3006
void GL_DynamicLight4f(GLfloat x, GLfloat y,GLfloat z,float fRed,float fGreen,float fBlue,GLfloat fAlpha);
void GL_StaticLight4f(GLfloat fRed,GLfloat fGreen,GLfloat fBlue,GLfloat fAlpha);

/*
 * draw normal
 *
 */
#if 0
void _md2_drawModel (md2_model_t *model, int frame,float fRed,float fGreen,float fBlue,GLfloat fAlpha)
{
	int i;

	md2_frame_t *f = &model->frames[frame];

	if (g_bMD2Light.value==1)
	{	
		GL_DynamicLight4f(xC,yC,zC,fRed,fGreen,fBlue,fAlpha);
	}
	else if (!g_bMD2Light.value)
		GL_StaticLight4f(fRed,fGreen,fBlue,fAlpha);

	glBegin (GL_TRIANGLES);

	for (i = 0; i < model->header.numTriangles; i++)
	{
		md2_triangle_t *t = &model->triangles[i];
		//int lightNormalIndex;
/*
		lightNormalIndex = f->vertices[t->vertexIndices[0]].lightNormalIndex;
		glNormal3f (avertexnormals[lightNormalIndex][0],
			avertexnormals[lightNormalIndex][2],
			-avertexnormals[lightNormalIndex][1]);
*/
		glNormal3f (f->vertices[t->vertexIndices[0]].normal[0],
			f->vertices[t->vertexIndices[0]].normal[2],
			-f->vertices[t->vertexIndices[0]].normal[1]);
		glTexCoord2f ((float) model->texCoords[t->textureIndices[0]].s / (float) model->header.skinWidth,
					(float) model->texCoords[t->textureIndices[0]].t / (float) model->header.skinHeight);
		glVertex3f (f->vertices[t->vertexIndices[0]].vertex[0],
					f->vertices[t->vertexIndices[0]].vertex[1],
					f->vertices[t->vertexIndices[0]].vertex[2]);

/*
		lightNormalIndex = f->vertices[t->vertexIndices[1]].lightNormalIndex;
		glNormal3f (avertexnormals[lightNormalIndex][0],
			avertexnormals[lightNormalIndex][2],
			-avertexnormals[lightNormalIndex][1]);
*/
		glNormal3f (f->vertices[t->vertexIndices[1]].normal[0],
			f->vertices[t->vertexIndices[1]].normal[2],
			-f->vertices[t->vertexIndices[1]].normal[1]);
		glTexCoord2f ((float) model->texCoords[t->textureIndices[1]].s / (float) model->header.skinWidth,
					(float) model->texCoords[t->textureIndices[1]].t / (float) model->header.skinHeight);
		glVertex3f (f->vertices[t->vertexIndices[1]].vertex[0],
					f->vertices[t->vertexIndices[1]].vertex[1],
					f->vertices[t->vertexIndices[1]].vertex[2]);
/*
		lightNormalIndex = f->vertices[t->vertexIndices[2]].lightNormalIndex;
		glNormal3f (avertexnormals[lightNormalIndex][0],
			avertexnormals[lightNormalIndex][2],
			-avertexnormals[lightNormalIndex][1]);
*/
		glNormal3f (f->vertices[t->vertexIndices[2]].normal[0],
			f->vertices[t->vertexIndices[2]].normal[2],
			-f->vertices[t->vertexIndices[2]].normal[1]);
		glTexCoord2f ((float) model->texCoords[t->textureIndices[2]].s / (float) model->header.skinWidth,
					(float) model->texCoords[t->textureIndices[2]].t / (float) model->header.skinHeight);
		glVertex3f (f->vertices[t->vertexIndices[2]].vertex[0],
					f->vertices[t->vertexIndices[2]].vertex[1],
					f->vertices[t->vertexIndices[2]].vertex[2]);
	}
	glEnd ();
}



/*
 * draw interpolated
 *
 */
void _md2_drawModeli (md2_model_t *model, int frame1, int frame2, float pol)
{
	int i;

	md2_frame_t *f1 = &model->frames[frame1];
	md2_frame_t *f2 = &model->frames[frame2];

	glBegin (GL_TRIANGLES);

	for (i = 0; i < model->header.numTriangles; i++)
	{
		md2_triangle_t *t = &model->triangles[i];
		//int lightNormalIndex1, lightNormalIndex2;
		float x1, y1, z1, x2, y2, z2;
		float *n1, *n2;
		
		glTexCoord2f ((float) model->texCoords[t->textureIndices[0]].s / (float) model->header.skinWidth,
					(float) model->texCoords[t->textureIndices[0]].t / (float) model->header.skinHeight);
/*
		lightNormalIndex1 = f1->vertices[t->vertexIndices[0]].lightNormalIndex;
		lightNormalIndex2 = f2->vertices[t->vertexIndices[0]].lightNormalIndex;
		glNormal3f ((1.0f - pol) * avertexnormals[lightNormalIndex1][0] + pol * avertexnormals[lightNormalIndex2][0],
			(1.0f - pol) * avertexnormals[lightNormalIndex1][2] + pol * avertexnormals[lightNormalIndex2][2],
			(1.0f - pol) * -avertexnormals[lightNormalIndex1][1] + pol * -avertexnormals[lightNormalIndex2][1]);
*/
		n1 = f1->vertices[t->vertexIndices[0]].normal;
		n2 = f2->vertices[t->vertexIndices[0]].normal;
		glNormal3f ((1.0f - pol) * n1[0] + pol * n2[0],
			(1.0f - pol) * n1[2] + pol * n2[2],
			(1.0f - pol) * -n1[1] + pol * -n2[1]);

		x1 = f1->vertices[t->vertexIndices[0]].vertex[0];
		y1 = f1->vertices[t->vertexIndices[0]].vertex[1];
		z1 = f1->vertices[t->vertexIndices[0]].vertex[2];
		x2 = f2->vertices[t->vertexIndices[0]].vertex[0];
		y2 = f2->vertices[t->vertexIndices[0]].vertex[1];
		z2 = f2->vertices[t->vertexIndices[0]].vertex[2];
		glVertex3f (x1 + pol * (x2 - x1),
			y1 + pol * (y2 - y1),
			z1 + pol * (z2 - z1));

		glTexCoord2f ((float) model->texCoords[t->textureIndices[1]].s / (float) model->header.skinWidth,
					(float) model->texCoords[t->textureIndices[1]].t / (float) model->header.skinHeight);
/*
		lightNormalIndex1 = f1->vertices[t->vertexIndices[1]].lightNormalIndex;
		lightNormalIndex2 = f2->vertices[t->vertexIndices[1]].lightNormalIndex;
		glNormal3f ((1.0f - pol) * avertexnormals[lightNormalIndex1][0] + pol * avertexnormals[lightNormalIndex2][0],
			(1.0f - pol) * avertexnormals[lightNormalIndex1][2] + pol * avertexnormals[lightNormalIndex2][2],
			(1.0f - pol) * -avertexnormals[lightNormalIndex1][1] + pol * -avertexnormals[lightNormalIndex2][1]);
*/
		n1 = f1->vertices[t->vertexIndices[1]].normal;
		n2 = f2->vertices[t->vertexIndices[1]].normal;
		glNormal3f ((1.0f - pol) * n1[0] + pol * n2[0],
			(1.0f - pol) * n1[2] + pol * n2[2],
			(1.0f - pol) * -n1[1] + pol * -n2[1]);

		x1 = f1->vertices[t->vertexIndices[1]].vertex[0];
		y1 = f1->vertices[t->vertexIndices[1]].vertex[1];
		z1 = f1->vertices[t->vertexIndices[1]].vertex[2];
		x2 = f2->vertices[t->vertexIndices[1]].vertex[0];
		y2 = f2->vertices[t->vertexIndices[1]].vertex[1];
		z2 = f2->vertices[t->vertexIndices[1]].vertex[2];
		glVertex3f (x1 + pol * (x2 - x1),
			y1 + pol * (y2 - y1),
			z1 + pol * (z2 - z1));

		glTexCoord2f ((float) model->texCoords[t->textureIndices[2]].s / (float) model->header.skinWidth,
					(float) model->texCoords[t->textureIndices[2]].t / (float) model->header.skinHeight);
/*
		lightNormalIndex1 = f1->vertices[t->vertexIndices[2]].lightNormalIndex;
		lightNormalIndex2 = f2->vertices[t->vertexIndices[2]].lightNormalIndex;
		glNormal3f ((1.0f - pol) * avertexnormals[lightNormalIndex1][0] + pol * avertexnormals[lightNormalIndex2][0],
			(1.0f - pol) * avertexnormals[lightNormalIndex1][2] + pol * avertexnormals[lightNormalIndex2][2],
			(1.0f - pol) * -avertexnormals[lightNormalIndex1][1] + pol * -avertexnormals[lightNormalIndex2][1]);
*/
		n1 = f1->vertices[t->vertexIndices[2]].normal;
		n2 = f2->vertices[t->vertexIndices[2]].normal;
		glNormal3f ((1.0f - pol) * n1[0] + pol * n2[0],
			(1.0f - pol) * n1[2] + pol * n2[2],
			(1.0f - pol) * -n1[1] + pol * -n2[1]);

		x1 = f1->vertices[t->vertexIndices[2]].vertex[0];
		y1 = f1->vertices[t->vertexIndices[2]].vertex[1];
		z1 = f1->vertices[t->vertexIndices[2]].vertex[2];
		x2 = f2->vertices[t->vertexIndices[2]].vertex[0];
		y2 = f2->vertices[t->vertexIndices[2]].vertex[1];
		z2 = f2->vertices[t->vertexIndices[2]].vertex[2];
		glVertex3f (x1 + pol * (x2 - x1),
			y1 + pol * (y2 - y1),
			z1 + pol * (z2 - z1));
	}
	glEnd ();
}
#endif	// 0


/*
 * draw with glcommands
 *
 */	
void _md2_drawModelg (md2_model_t *model, int frame,float fRed,float fGreen,float fBlue,GLfloat fAlpha,fixed_t fDist)
{
	int i = 0;
	int val = model->glCommandBuffer[i++];
	float x,y,z,x2,z2;

	if (frame>=model->header.numFrames)
		frame=model->header.numFrames-1;

	if (g_bMD2Light.value==1)
	{	
		GL_DynamicLight4f(xC,yC,zC,fRed,fGreen,fBlue,fAlpha);
	}
	else if (!g_bMD2Light.value)
		GL_StaticLight4f(fRed,fGreen,fBlue,fAlpha);

	while (val != 0)
	{
		int count;

		if (val > 0)
		{
			glBegin (GL_TRIANGLE_STRIP);
			count = val;
		}
		else
		{
			glBegin (GL_TRIANGLE_FAN);
			count = -val;
		}

	if (g_bMD2Light.value==2)
	{	GL_BuildLightsListForMD2(xC,yC,zC);	// MR1602
		while (count--)
		{
			float s = *(float *) &model->glCommandBuffer[i++];
			float t = *(float *) &model->glCommandBuffer[i++];
			int index = model->glCommandBuffer[i++];
			int lightNormalIndex = model->frames[frame].vertices[index].lightNormalIndex;

			// MR3006: DynLight
			x=model->frames[frame].vertices[index].vertex[0];
			y=model->frames[frame].vertices[index].vertex[1]+yC;
			z=model->frames[frame].vertices[index].vertex[2];
			x2=x*cosA-z*sinA+xC;
			z2=x*sinA+z*cosA+zC;
			// MR1602
			//GL_DynamicLight4f(x2,y,z2,fRed,fGreen,fBlue,fAlpha);
			//GL_MD2DynamicLight4f(x2,y,z2,fRed,fGreen,fBlue,fAlpha,avertexnormals[lightNormalIndex]);
			GL_MD2DynamicLight4f(x2,y,z2,fRed,fGreen,fBlue,fAlpha,model->frames[frame].vertices[index].normal);

			glTexCoord2f (s, t);
/*
			glNormal3f (avertexnormals[lightNormalIndex][0],
				avertexnormals[lightNormalIndex][2],
				-avertexnormals[lightNormalIndex][1]);
*/
// MR... a voir avec dynamic lighting
/*			glNormal3f (model->frames[frame].vertices[index].normal[0],
				model->frames[frame].vertices[index].normal[2],
				-model->frames[frame].vertices[index].normal[1]);*/
			glVertex3f (x,model->frames[frame].vertices[index].vertex[1],z);
		}
	}
	else
	{	while (count--)
		{
			float s = *(float *) &model->glCommandBuffer[i++];
			float t = *(float *) &model->glCommandBuffer[i++];
			int index = model->glCommandBuffer[i++];

			glTexCoord2f (s, t);
			glVertex3f (model->frames[frame].vertices[index].vertex[0],
				model->frames[frame].vertices[index].vertex[1],
				model->frames[frame].vertices[index].vertex[2]);
		}
	}
		glEnd ();

		val = model->glCommandBuffer[i++];
	}
}



/*
 * draw with glcommands, interpolated
 *
 */
void _md2_drawModelgi (md2_model_t *model, int frame1, int frame2, float pol,GLfloat fRed,GLfloat fGreen,GLfloat fBlue,GLfloat fAlpha,fixed_t fDist)
{
	int i = 0;
	int val = model->glCommandBuffer[i++];

/*	if (g_bMD2Light.value==1)
	{	
		GL_DynamicLight4f(xC,yC,zC,fRed,fGreen,fBlue,fAlpha);
	}
	else if (!g_bMD2Light.value)
		GL_StaticLight4f(fRed,fGreen,fBlue,fAlpha);*/

	if (frame2>=model->header.numFrames)
	{	
		if (frame1>=model->header.numFrames)
			frame1=model->header.numFrames-1;
		frame2=frame1;
		pol=0.0f;
	}
	while (val != 0)
	{
		int count;

		if (val > 0)
		{
			glBegin (GL_TRIANGLE_STRIP);
			count = val;
		}
		else
		{
			glBegin (GL_TRIANGLE_FAN);
			count = -val;
		}

	if ((g_bMD2Light.value==2)&&(fDist<1000000*(fixed_t)g_iMD2VLightDist.value))		// MR231200 (fDist)
	{ float y;

		GL_BuildLightsListForMD2(xC,yC,zC);
		while (count--)
		{
			float s = *(float *) &model->glCommandBuffer[i++];
			float t = *(float *) &model->glCommandBuffer[i++];
			int index = model->glCommandBuffer[i++];
			float x1, y1, z1, x2, y2, z2;
			float *n1 = model->frames[frame1].vertices[index].normal;
			float *n2 = model->frames[frame2].vertices[index].normal;
			float n3[3];

			x1=model->frames[frame1].vertices[index].vertex[0];
			y=model->frames[frame1].vertices[index].vertex[1]+yC;
			z1=model->frames[frame1].vertices[index].vertex[2];
			x2=x1*cosA-z1*sinA+xC;
			z2=x1*sinA+z1*cosA+zC;
			n3[0]=(1.0f - pol) * n1[0] + pol * n2[0];
			n3[1]=(1.0f - pol) * n1[1] + pol * n2[1];
			n3[2]=(1.0f - pol) * n1[2] + pol * n2[2];
			GL_MD2DynamicLight4f(x2,y,z2,fRed,fGreen,fBlue,fAlpha,n3);
			//GL_DynamicLight4f(x2,y,z2,fRed,fGreen,fBlue,fAlpha);

			/* texcoords don't need to be interpolated */
			glTexCoord2f (s, t);

			/* interpolate light normal */
/*			glNormal3f ((1.0f - pol) * n1[0] + pol * n2[0],
				(1.0f - pol) * n1[2] + pol * n2[2],
				(1.0f - pol) * -n1[1] + pol * -n2[1]);*/

			/* interpolate vertices */
			//x1 = model->frames[frame1].vertices[index].vertex[0];
			y1 = model->frames[frame1].vertices[index].vertex[1];
			//z1 = model->frames[frame1].vertices[index].vertex[2];

			x2 = model->frames[frame2].vertices[index].vertex[0];
			y2 = model->frames[frame2].vertices[index].vertex[1];
			z2 = model->frames[frame2].vertices[index].vertex[2];

			glVertex3f (x1 + pol * (x2 - x1),
				y1 + pol * (y2 - y1),
				z1 + pol * (z2 - z1));
		}
	}
	else
	{	
		if (g_bMD2Light.value)
		{	
			GL_DynamicLight4f(xC,yC,zC,fRed,fGreen,fBlue,fAlpha);
		}
		else
			GL_StaticLight4f(fRed,fGreen,fBlue,fAlpha);

		while (count--)
		{
			float s = *(float *) &model->glCommandBuffer[i++];
			float t = *(float *) &model->glCommandBuffer[i++];
			int index = model->glCommandBuffer[i++];
			float x1, y1, z1, x2, y2, z2;

			/* texcoords don't need to be interpolated */
			glTexCoord2f (s, t);

			/* interpolate vertices */
			x1 = model->frames[frame1].vertices[index].vertex[0];
			y1 = model->frames[frame1].vertices[index].vertex[1];
			z1 = model->frames[frame1].vertices[index].vertex[2];

			x2 = model->frames[frame2].vertices[index].vertex[0];
			y2 = model->frames[frame2].vertices[index].vertex[1];
			z2 = model->frames[frame2].vertices[index].vertex[2];

			glVertex3f (x1 + pol * (x2 - x1),
				y1 + pol * (y2 - y1),
				z1 + pol * (z2 - z1));
		}
	}

		glEnd ();

		val = model->glCommandBuffer[i++];
	}
}


void _makeFacetNormal (md2_model_t *model, md2_triangle_t *t, int frame, float *fn)
{
	float v1[3], v2[3];

	v1[0] = model->frames[frame].vertices[t->vertexIndices[1]].vertex[0] - model->frames[frame].vertices[t->vertexIndices[0]].vertex[0];
	v1[1] = model->frames[frame].vertices[t->vertexIndices[1]].vertex[1] - model->frames[frame].vertices[t->vertexIndices[0]].vertex[1];
	v1[2] = model->frames[frame].vertices[t->vertexIndices[1]].vertex[2] - model->frames[frame].vertices[t->vertexIndices[0]].vertex[2];

	v2[0] = model->frames[frame].vertices[t->vertexIndices[2]].vertex[0] - model->frames[frame].vertices[t->vertexIndices[0]].vertex[0];
	v2[1] = model->frames[frame].vertices[t->vertexIndices[2]].vertex[1] - model->frames[frame].vertices[t->vertexIndices[0]].vertex[1];
	v2[2] = model->frames[frame].vertices[t->vertexIndices[2]].vertex[2] - model->frames[frame].vertices[t->vertexIndices[0]].vertex[2];

	fn[0] = v1[1] * v2[2] - v1[2] * v2[1];
	fn[1] = v1[2] * v2[0] - v1[0] * v2[2];
	fn[2] = v1[0] * v2[1] - v1[1] * v2[0];
}



void _normalize (float *n)
{
	float l = (float) sqrt (n[0] * n[0] + n[1] * n[1] + n[2] * n[2]);

	if (l != 0.0f)
	{
		n[0] /= l;
		n[1] /= l;
		n[2] /= l;
	}
}



void md2_generateLightNormals (md2_model_t *model)
{
	int i;

	if (!model)
		return;


	for (i = 0; i < model->header.numFrames; i++)
	{
		int j;

		/* clear all normals */
		for (j = 0; j < model->header.numVertices; j++)
		{
			model->frames[i].vertices[j].normal[0] = 0.0f;
			model->frames[i].vertices[j].normal[1] = 0.0f;
			model->frames[i].vertices[j].normal[2] = 0.0f;
		}

		/* calc normals */
		for (j = 0; j < model->header.numTriangles; j++)
		{
			int k;
			float facetnormal[3];

			_makeFacetNormal (model, &model->triangles[j], i, facetnormal);

			for (k = 0; k < 3; k++)
			{
				model->frames[i].vertices[model->triangles[j].vertexIndices[k]].normal[0] -= facetnormal[0];
				model->frames[i].vertices[model->triangles[j].vertexIndices[k]].normal[1] += facetnormal[2];
				model->frames[i].vertices[model->triangles[j].vertexIndices[k]].normal[2] -= facetnormal[1];
			}
		}

		/* normalize normals */
		for (j = 0; j < model->header.numVertices; j++)
			_normalize (model->frames[i].vertices[j].normal);
	}
}



int md2_getAnimationCount (md2_model_t *model)
{
	int i, j, pos;
	int count;
	int lastId;
	char name[16], last[16];

	strcpy (last, model->frames[0].name);
	pos = strlen (last) - 1;
	j = 0;
	while (last[pos] >= '0' && last[pos] <= '9' && j < 2)
	{
		pos--;
		j++;
	}
	last[pos + 1] = '\0';

	lastId = 0;
	count = 0;

	for (i = 0; i <= model->header.numFrames; i++)
	{
		if (i == model->header.numFrames)
			strcpy (name, ""); // some kind of a sentinel
		else
			strcpy (name, model->frames[i].name);
		pos = strlen (name) - 1;
		j = 0;
		while (name[pos] >= '0' && name[pos] <= '9' && j < 2)
		{
			pos--;
			j++;
		}
		name[pos + 1] = '\0';

		if (strcmp (last, name))
		{
			strcpy (last, name);
			count++;
		}
	}

	return count;
}



const char *md2_getAnimationName (md2_model_t *model, int animation)
{
	int i, j, pos;
	int count;
	int lastId;
	static char last[32];
	char name[32];

	strcpy (last, model->frames[0].name);
	pos = strlen (last) - 1;
	j = 0;
	while (last[pos] >= '0' && last[pos] <= '9' && j < 2)
	{
		pos--;
		j++;
	}
	last[pos + 1] = '\0';

	lastId = 0;
	count = 0;

	for (i = 0; i <= model->header.numFrames; i++)
	{
		if (i == model->header.numFrames)
			strcpy (name, ""); // some kind of a sentinel
		else
			strcpy (name, model->frames[i].name);
		pos = strlen (name) - 1;
		j = 0;
		while (name[pos] >= '0' && name[pos] <= '9' && j < 2)
		{
			pos--;
			j++;
		}
		name[pos + 1] = '\0';

		if (strcmp (last, name))
		{
			if (count == animation)
				return last;

			strcpy (last, name);
			count++;
		}
	}

	return 0;
}



void md2_getAnimationFrames (md2_model_t *model, int animation, int *startFrame, int *endFrame)
{
	int i, j, pos;
	int count, numFrames, frameCount;
	int lastId;
	char name[16], last[16];

	strcpy (last, model->frames[0].name);
	pos = strlen (last) - 1;
	j = 0;
	while (last[pos] >= '0' && last[pos] <= '9' && j < 2)
	{
		pos--;
		j++;
	}
	last[pos + 1] = '\0';

	lastId = 0;
	count = 0;
	numFrames = 0;
	frameCount = 0;

	for (i = 0; i <= model->header.numFrames; i++)
	{
		if (i == model->header.numFrames)
			strcpy (name, ""); // some kind of a sentinel
		else
			strcpy (name, model->frames[i].name);
		pos = strlen (name) - 1;
		j = 0;
		while (name[pos] >= '0' && name[pos] <= '9' && j < 2)
		{
			pos--;
			j++;
		}
		name[pos + 1] = '\0';

		if (strcmp (last, name))
		{
			strcpy (last, name);

			if (count == animation)
			{
				*startFrame = frameCount - numFrames;
				*endFrame = frameCount - 1;
				return;
			}

			count++;
			numFrames = 0;
		}
		frameCount++;
		numFrames++;
	}


	*startFrame = *endFrame = 0;
}


#endif	// ZDOOM_GL